////////////////////////////////////////////////////////////////////////////////
//                                 APICulture
//                        (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                               
////////////////////////////////////////////////////////////////////////////////

unit AclControlFire;

interface

uses
  Windows, Messages, WinLite;

type
  TControlFire = class(TLiteWindow)
  private
    FParticles  : array of TRect;
    FPartSpeed  : Integer;
    FPartSize   : Integer;
    FPartCount  : Integer;
    FDrawTimer  : Integer;
    FDrawBuffer : THandle;
    FDrawContext: THandle;
    FWidth      : Integer;
    FHeight     : Integer;
  protected
    procedure   CreateWindowParams(var WindowParams: TWindowParams); override;
  protected
    procedure   OnEraseBackground(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure   OnPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure   OnCreate(var Msg: TWMCreate); message WM_CREATE;
    procedure   OnDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure   OnTimer(var Msg: TWMTimer); message WM_TIMER;
    procedure   OnSize(var Msg: TWMSize); message WM_SIZE;
  protected
    procedure   DrawBuffer;
    procedure   DrawBufferCreate;
    procedure   DrawBufferDestroy;
  public
    procedure   PartArrayCreate(Count, Speed, Size: Integer);
    procedure   PartArrayUpdate;
    procedure   PartArrayDraw;
  end;

implementation

{$I res\resource.inc}

////////////////////////////////////////////////////////////////////////////////
// TControlFire
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.CreateWindowParams(var WindowParams: TWindowParams);
begin
  inherited;
  //   
  WindowParams.Caption := '';
  WindowParams.Style   := WS_CHILD;
  WindowParams.ExStyle := 0;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnCreate(var Msg: TWMCreate);
begin
  inherited;
  //  
  FDrawTimer := SetTimer(WndHandle, 1, 50, nil);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnDestroy(var Msg: TWMDestroy);
begin
  //  
  KillTimer(WndHandle, FDrawTimer);
  //      
  DrawBufferDestroy;
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnSize(var Msg: TWMSize);
begin
  //   
  FWidth  := Msg.Width;
  FHeight := Msg.Height;
  //      
  DrawBufferDestroy;
  //      
  DrawBufferCreate;
  //   
  DrawBuffer;
end;

////////////////////////////////////////////////////////////////////////////////
// TControlFire
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnEraseBackground(var Msg: TWMEraseBkgnd);
begin
  //  - ,    -      , 
  //  
  Msg.Result := 1;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnPaint(var Msg: TWMPaint);
var
  PS : TPaintStruct;
begin
  //  
  BeginPaint(WndHandle, PS);
  //        
  BitBlt(
    PS.hdc, PS.rcPaint.Left, PS.rcPaint.Top, PS.rcPaint.Right-PS.rcPaint.Left, PS.rcPaint.Bottom-PS.rcPaint.Top,
    FDrawContext, PS.rcPaint.Left, PS.rcPaint.Top,
    SRCCOPY
  );
  //  
  EndPaint(WndHandle, PS);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.OnTimer(var Msg: TWMTimer);
var
  KillMsg : TMsg;
begin
  //   
  PartArrayUpdate;
  //  
  DrawBuffer;
  //    ,   
  if IsWindowVisible(WndHandle) then
    InvalidateRect(WndHandle, nil, False);
  //     
  repeat until not PeekMessage(KillMsg, WndHandle, WM_TIMER, WM_TIMER, PM_REMOVE);
end;

////////////////////////////////////////////////////////////////////////////////
// TControlFire
// _____________________________________________________________________________
//    
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.DrawBuffer;
var
  Rect : TRect;
begin
  //   
  Rect.Left   := 0;
  Rect.Top    := 0;
  Rect.Right  := FWidth;
  Rect.Bottom := FHeight;
  //  
  FillRect(FDrawContext, Rect, GetStockObject(BLACK_BRUSH));
  //  
  PartArrayDraw;
  //  
  DrawEdge(FDrawContext, Rect, BDR_SUNKENOUTER, BF_RECT);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.DrawBufferCreate;
var
  DC : THandle;
begin
  //   
  DC := GetDC(WndHandle);
  //    
  FDrawBuffer  := CreateCompatibleBitmap(DC, FWidth, FHeight);
  //    
  FDrawContext := CreateCompatibleDC(DC);
  //       
  SelectObject(FDrawContext, FDrawBuffer);
  //   
  ReleaseDC(WndHandle, DC);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.DrawBufferDestroy;
begin
  //   ,   
  if FDrawBuffer <> 0 then DeleteObject(FDrawBuffer);
  FDrawBuffer := 0;
  //   ,   
  if FDrawContext <> 0 then DeleteDC(FDrawContext);
  FDrawContext := 0;
end;

////////////////////////////////////////////////////////////////////////////////
// TControlFire
// _____________________________________________________________________________
//    
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.PartArrayCreate(Count, Speed, Size: Integer);
var
  i : Integer;
begin
  //   
  FPartCount := Count;
  FPartSpeed := Speed;
  FPartSize  := Size;
  //   
  SetLength(FParticles, Count);
  //  
  for i:=0 to Length(FParticles)-1 do
  begin
    //  
    FParticles[i].Left   := (FWidth div 2) + Random(5) - 2;
    FParticles[i].Top    := FHeight-Random(3);
    FParticles[i].Right  := 0;
    FParticles[i].Bottom := -Random(FPartSpeed)+1;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.PartArrayUpdate;
var
  i : Integer;
begin
  //    
  for i:=0 to Length(FParticles)-1 do
  begin
    //    
    FParticles[i].Left   := FParticles[i].Left   + FParticles[i].Right;
    FParticles[i].Top    := FParticles[i].Top    + FParticles[i].Bottom;
    FParticles[i].Right  := FParticles[i].Right  + Random(5) - 2;
    FParticles[i].Bottom := FParticles[i].Bottom + Random(3);
    //       
    if (FParticles[i].Left > FWidth) or (FParticles[i].Top > FHeight) or
       (FParticles[i].Left < 0)      or (FParticles[i].Top < 0) then
    begin
      //  
      FParticles[i].Left   := (FWidth div 2) + Random(5) - 2;
      FParticles[i].Top    := FHeight-Random(3);
      FParticles[i].Right  := 0;
      FParticles[i].Bottom := -Random(FPartSpeed)+1;
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlFire.PartArrayDraw;
var
  i : Integer;
begin
  //    
  SelectObject(FDrawContext, GetStockObject(GRAY_BRUSH));
  SelectObject(FDrawContext, GetStockObject(WHITE_PEN));
  //    
  for i:=0 to Length(FParticles)-1 do
  begin
    Rectangle(FDrawContext, FParticles[i].Left, FParticles[i].Top,
      FParticles[i].Left+FPartSize, FParticles[i].Top+FPartSize);
  end;
end;

end.
