////////////////////////////////////////////////////////////////////////////////
//                                 APICulture
//                        (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                             OpenGL
////////////////////////////////////////////////////////////////////////////////

unit AclControlOpenGL;

interface

uses
  Windows, Messages, WinLite, OpenGL;

type
  TControlOpenGL = class(TLiteWindow)
  private
    FUpdateTimer: Integer;
    FContextDC  : THandle;
    FContextGL  : THandle;
    FRotateAngle: Single;
    FPalette    : THandle;
    FElemX      : array [0..5] of Single;
    FElemY      : array [0..5] of Single;
  protected
    procedure   CreateWindowParams(var WindowParams: TWindowParams); override;
  protected
    procedure   OnEraseBackground(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure   OnCreate(var Msg: TWMCreate); message WM_CREATE;
    procedure   OnDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure   OnSize(var Msg: TWMSize); message WM_SIZE;
    procedure   OnTimer(var Msg: TWMTimer); message WM_TIMER;
    procedure   OnPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure   OnQueryNewPalette(var Msg: TWMQueryNewPalette); message WM_QUERYNEWPALETTE;
    procedure   OnPaletteChanged(var Msg: TWMPaletteChanged); message WM_PALETTECHANGED;
  protected
    procedure   SetDCPixelFormat;
  end;

implementation

{$I res\resource.inc}

////////////////////////////////////////////////////////////////////////////////
// TControlOpenGL
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.CreateWindowParams(var WindowParams: TWindowParams);
begin
  inherited;
  //   
  WindowParams.Caption := '';
  WindowParams.Style   := WS_CHILD or WS_CLIPSIBLINGS or WS_CLIPCHILDREN;
  WindowParams.ExStyle := 0;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnCreate(var Msg: TWMCreate);
const
  Texture: array [0..3] of Single = (0.3, 0.3, 0.5, 0.0);
var
  i : Integer;
begin
  inherited;
  //  
  FUpdateTimer := SetTimer(WndHandle, 1, 50, nil);
  //   
  FContextDC := GetDC(WndHandle);
  //   
  SetDCPixelFormat;
  //  GL-
  FContextGL := wglCreateContext(FContextDC);
  //   
  wglMakeCurrent(FContextDC, FContextGL);
  //  
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @Texture);
  //   
  FRotateAngle := 0;
  for i:=0 to 5 do
  begin
    FElemX[i] := Sin(Pi / 3 * i);
    FElemY[i] := Cos(Pi / 3 * i);
  end;
  //  
  glNewList(Cardinal(Self), GL_COMPILE);
    glScalef(0.25, 0.25, 0.25);
    glBegin(GL_POLYGON);
      glNormal3f(0.0, 0.0, 1.0);
      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(-1.0, 1.0, 1.0);
      glVertex3f(-1.0, -1.0, 1.0);
      glVertex3f(1.0, -1.0, 1.0);
    glEnd;
    glBegin(GL_POLYGON);
      glNormal3f(-1.0, 0.0, 0.0);
      glVertex3f(-1.0, 1.0, 1.0);
      glVertex3f(-1.0, 1.0, -1.0);
      glVertex3f(-1.0, -1.0, -1.0);
      glVertex3f(-1.0, -1.0, 1.0);
    glEnd;
    glBegin(GL_POLYGON);
      glNormal3f(1.0, 0.0, 0.0);
      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(1.0, -1.0, 1.0);
      glVertex3f(1.0, -1.0, -1.0);
      glVertex3f(1.0, 1.0, -1.0);
    glEnd;
    glBegin(GL_POLYGON);
      glNormal3f(0.0, 1.0, 0.0);
      glVertex3f(-1.0, 1.0, -1.0);
      glVertex3f(-1.0, 1.0, 1.0);
      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(1.0, 1.0, -1.0);
    glEnd;
    glBegin(GL_POLYGON);
      glNormal3f(0.0, -1.0, 0.0);
      glVertex3f(-1.0, -1.0, -1.0);
      glVertex3f(1.0, -1.0, -1.0);
      glVertex3f(1.0, -1.0, 1.0);
      glVertex3f(-1.0, -1.0, 1.0);
    glEnd;
    glScalef(4, 4, 4);
  glEndList;
  glClearColor(0.25, 0.1, 0.25, 0.0);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnDestroy(var Msg: TWMDestroy);
begin
  //  
  glDeleteLists(Cardinal(Self), 1);
  //   
  wglMakeCurrent(0, 0);
  //  GL-
  wglDeleteContext(FContextGL);
  //   
  ReleaseDC(WndHandle, FContextDC);
  //  
  KillTimer(WndHandle, FUpdateTimer);
  //   
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
// TControlOpenGL
// _____________________________________________________________________________
// 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnSize(var Msg: TWMSize);
begin
  //   
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(18.0, Msg.Width / Msg.Height, 6.0, 10.0);
  glViewport(0, 0, Msg.Width, Msg.Height);
  glMatrixMode(GL_MODELVIEW);
  //  
  InvalidateRect(WndHandle, nil, False);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnTimer(var Msg: TWMTimer);
var
  KillMsg : TMsg;
begin
  //   
  FRotateAngle := FRotateAngle + 6;
  If (FRotateAngle >= 360.0) then FRotateAngle := 0.0;
  //  
  InvalidateRect(WndHandle, nil, False);
  //     
  repeat until not PeekMessage(KillMsg, WndHandle, WM_TIMER, WM_TIMER, PM_REMOVE);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnEraseBackground(var Msg: TWMEraseBkgnd);
begin
  //  - ,    -      , 
  //  
  Msg.Result := 1;
end;

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnPaint(var Msg: TWMPaint);
var
  PS   : TPaintStruct;
  i    : Integer;
begin
  //  
  BeginPaint(WndHandle, ps);
  //  GL
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;
  glTranslatef(0.0, 0.0, -9.0);
  glRotatef(60.0, 0.0, 0.0, 1.0);
  // 
  glRotatef(FRotateAngle, 2.0, 0.0, 1.0);
  //   
  for i:=0 to 5 do
  begin
    glPushMatrix;
    glTranslatef(FElemX[i], FElemY[i], 0.0);
    glRotatef(-60 * i + FRotateAngle, 0.0, 0.0, 1.0);
    glCallList(Cardinal(Self));
    glPopMatrix;                   
  end;
  //   
  SwapBuffers(FContextDC);                
  //  
  EndPaint(WndHandle, ps);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnPaletteChanged(var Msg: TWMPaletteChanged);
begin
  //    ,  - 
  //   
  if (FPalette <> 0) and (THandle(TMessage(Msg).wParam) <> WndHandle) then
  begin
    if RealizePalette(FContextDC) <> GDI_ERROR then UpdateColors(FContextDC);
    // 
    Msg.Result := 0;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.OnQueryNewPalette(var Msg: TWMQueryNewPalette);
begin
  if FPalette <> 0 then
  begin
    // 
    Msg.Result := RealizePalette(FContextDC);
    //  
    if DWord(Msg.Result) <> GDI_ERROR then InvalidateRect(WndHandle, nil, False);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// TControlOpenGL
// _____________________________________________________________________________
//   c OpenGL
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TControlOpenGL.SetDCPixelFormat;
var
  RedMask  : Byte;
  GreenMask: Byte;
  BlueMask : Byte;
  Palette  : PLogPalette;
  Heap     : THandle;
  Colors   : Integer;
  i        : Integer;
  PixelFmt : Integer;
  PFD      : TPixelFormatDescriptor;
begin
  //    
  FillChar(PFD, SizeOf(PFD), 0);
  with PFD do
  begin
    nSize      := SizeOf(PFD);
    nVersion   := 1;
    dwFlags    := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
    iPixelType := PFD_TYPE_RGBA;
    cColorBits := 24;
    cDepthBits := 32;
    iLayerType := PFD_MAIN_PLANE;
  end;
  //     
  PixelFmt := ChoosePixelFormat(FContextDC, @PFD);
  SetPixelFormat(FContextDC, PixelFmt, @PFD);
  //   
  DescribePixelFormat(FContextDC, PixelFmt, SizeOf(TPixelFormatDescriptor), PFD);
  if (pfd.dwFlags and PFD_NEED_PALETTE) <> 0 then
  begin
    Colors    := 1 shl PFD.cColorBits;
    Heap      := GetProcessHeap;
    Palette   := HeapAlloc(Heap, 0, SizeOf(TLogPalette) + (Colors * SizeOf(TPaletteEntry)));
    //        
    Palette^.palVersion    := $300;
    Palette^.palNumEntries := Colors;
    RedMask   := (1 shl PFD.cRedBits) - 1;
    GreenMask := (1 shl PFD.cGreenBits) - 1;
    BlueMask  := (1 shl PFD.cBlueBits) - 1;
    //   
    for i:=0 to Colors - 1 do
    begin
      Palette^.palPalEntry[i].peRed   := (((i shr PFD.cRedShift)   and RedMask)   * 255) div RedMask;
      Palette^.palPalEntry[i].peGreen := (((i shr PFD.cGreenShift) and GreenMask) * 255) div GreenMask;
      Palette^.palPalEntry[i].peBlue  := (((i shr PFD.cBlueShift)  and BlueMask)  * 255) div BlueMask;
      Palette^.palPalEntry[i].peFlags := 0;
    end;
    //  
    FPalette := CreatePalette(Palette^);
    HeapFree(Heap, 0, Palette);
    //     
    if FPalette <> 0 then
    begin
      SelectPalette(FContextDC, FPalette, False);
      RealizePalette(FContextDC);
    end;
  end;
end;

end.
